from __future__ import absolute_import

import re


class Subscribers(object):
    # Subscribers base API uri
    base_api_url = "api/subscribers"

    def __init__(self, api_client):
        self.api_client = api_client

    def list(self, **kwargs):
        """
        Lists all subscribers

        Returns a list of all subscribers.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#list-all-subscribers

        :param **kwargs: dict You can pass additional arguments - page, limit or filter by status
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = ["list", "limit", "page"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)
            query_params[key] = val

        return self.api_client.request("GET", self.base_api_url, query_params).json()

    def create(self, email, **kwargs):
        """
        Create a subscriber

        Provides ability to create a new subscriber.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#create-update-subscriber

        :param email: string Valid email address as per RFC 2821
        :param **kwargs: dict You can pass additional arguments - fields, groups, status, subscribed_at, ip_address, opted_in_at, optin_ip and unsubscribed_at
        :raises: :class: `TypeError` : `email` is not a valid email address
        :raises: :class: `TypeError` : `fields` argument should be a dict
        :raises: :class: `TypeError` : `groups` argument should be a list
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = [
            "fields",
            "groups",
            "status",
            "subscribed_at",
            "ip_address",
            "opted_in_at",
            "optin_ip",
            "unsubscribed_at",
        ]

        valid = re.search(r"[\w.]+\@[\w.]+", email)

        if not valid:
            raise TypeError("`email` is not a valid email address.")

        params = locals()
        body_params = {"email": email}

        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)

            if key == "fields" and type(val) is not dict:
                raise TypeError("`fields` argument should be a dict.")

            if key == "groups" and type(val) is not list:
                raise TypeError("`groups` argument should be a list.")

            body_params[key] = val

        return self.api_client.request(
            "POST", self.base_api_url, body=body_params
        ).json()

    def update(self, email, **kwargs):
        """
        Update a subscriber

        Provides ability to update an existing subscriber.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#create-update-subscriber

        :param email: string Valid email address as per RFC 2821
        :param **kwargs: dict You can pass additional arguments - fields, groups, status, subscribed_at, ip_address, opted_in_at, optin_ip and unsubscribed_at
        :raises: :class: `TypeError` : `email` is not a valid email address
        :raises: :class: `TypeError` : `fields` argument should be a dict
        :raises: :class: `TypeError` : `groups` argument should be a list
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = [
            "fields",
            "groups",
            "status",
            "subscribed_at",
            "ip_address",
            "opted_in_at",
            "optin_ip",
            "unsubscribed_at",
        ]

        valid = re.search(r"[\w.]+\@[\w.]+", email)

        if not valid:
            raise TypeError("`email` is not a valid email address.")

        params = locals()
        body_params = {"email": email}

        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)

            if key == "fields" and type(val) is not dict:
                raise TypeError("`fields` argument should be a dict.")

            if key == "groups" and type(val) is not list:
                raise TypeError("`groups` argument should be a list.")

            body_params[key] = val

        return self.api_client.request(
            "POST", self.base_api_url, body=body_params
        ).json()

    def get(self, subscriber_id):
        """
        Get a subscriber

        Returns information about subscriber.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#fetch-a-subscriber

        :param subscriber_id: int/string Susbscriber email address or ID
        :raises: :class: `TypeError` : Provided email address or subscriber id are not valid
        :return: JSON array
        :rtype: dict
        """

        valid = re.search(r"[\w.]+\@[\w.]+", subscriber_id)

        if not valid and not isinstance(subscriber_id, int):
            raise TypeError("Provided email address or subscriber id are not valid.")

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{subscriber_id}"
        ).json()

    def delete(self, subscriber_id):
        """
        Delete a subscriber

        Provides ability to delete existing subscriber.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#delete-a-subscriber

        :param subscriber_id: int Susbscriber ID
        :raises: :class: `TypeError` : `subscriber_id` type is not valid
        :return: `true` if action was successful, `false` if subscriber was not found
        :rtype: bool
        """

        if not isinstance(subscriber_id, int):
            raise TypeError(
                f"`subscriber_id` type is not valid. Expected `int`, got {type(subscriber_id)}."
            )

        response = self.api_client.request(
            "DELETE", f"{self.base_api_url}/{subscriber_id}"
        )

        return response.status_code

    def get_import(self, import_id):
        """
        Get single import

        Returns a single import report.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#get-single-import

        :param import_id: int Import ID
        :raises: :class: `TypeError` : `import_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(import_id, int):
            raise TypeError(
                f"`import_id` type is not valid. Expected `int`, got {type(import_id)}."
            )

        return self.api_client.request(
            "GET", f"{self.base_api_url}/import/{import_id}"
        ).json()

    def assign_subscriber_to_group(self, subscriber_id, group_id):
        """
        Assign subscriber to a group

        Provides the ability to assign an existing subscriber to a group.
        Ref: https://developers.mailerlite.com/docs/groups.html#assign-subscriber-to-a-group

        :param subscriber_id: int Subscriber ID
        :param group_id: int Group ID
        :raises: :class: `TypeError` : `subscriber_id` type is not valid
        :raises: :class: `TypeError` : `group_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(subscriber_id, int):
            raise TypeError(
                f"`subscriber_id` type is not valid. Expected `int`, got {type(subscriber_id)}."
            )

        if not isinstance(group_id, int):
            raise TypeError(
                f"`group_id` type is not valid. Expected `int`, got {type(group_id)}."
            )

        return self.api_client.request(
            "POST", f"{self.base_api_url}/{subscriber_id}/groups/{group_id}"
        ).json()

    def unassign_subscriber_from_group(self, subscriber_id, group_id):
        """
        Unassign subscriber to a group

        Provides the ability to unassign an existing subscriber to a group.
        Ref: https://developers.mailerlite.com/docs/groups.html#unassign-subscriber-from-a-group

        :param subscriber_id: int Subscriber ID
        :param group_id: int Group ID
        :raises: :class: `TypeError` : `subscriber_id` type is not valid
        :raises: :class: `TypeError` : `group_id` type is not valid
        :return: `true` if action was successful, `false` if subscriber was not found
        :rtype: bool
        """

        if not isinstance(subscriber_id, int):
            raise TypeError(
                f"`subscriber_id` type is not valid. Expected `int`, got {type(subscriber_id)}."
            )

        if not isinstance(group_id, int):
            raise TypeError(
                f"`group_id` type is not valid. Expected `int`, got {type(group_id)}."
            )

        response = self.api_client.request(
            "DELETE", f"{self.base_api_url}/{subscriber_id}/groups/{group_id}"
        )

        return True if response.status_code == 204 else False

    def count(self):
        """
        Fetch total subscribers count

        Returns a total count of subscribers.
        Ref: https://developers.mailerlite.com/docs/subscribers.html#fetch-total-subscribers-count

        :return: JSON array
        :rtype: dict
        """

        return self.list(limit=0)



from __future__ import absolute_import


class Segments(object):
    # Segments base API uri
    base_api_url = "api/segments"

    def __init__(self, api_client):
        self.api_client = api_client

    def list(self, **kwargs):
        """
        Lists all segments

        Returns a list of all segments.
        Ref: https://developers.mailerlite.com/docs/segments.html#list-all-segments

        :param **kwargs: dict You can pass additional arguments - page and limit
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = ["limit", "page"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)
            query_params[key] = val

        return self.api_client.request("GET", self.base_api_url, query_params).json()

    def get_subscribers(self, segment_id, **kwargs):
        """
        Get subscribers belonging to a segment

        Returns a list of subscribers belonging to a segment.
        Ref: https://developers.mailerlite.com/docs/segments.html#get-subscribers-belonging-to-a-segment

        :param segment_id: int Segment ID
        :param **kwargs: dict You can pass additional arguments - after, limit or filter by status
        :raises: :class: `TypeError` : Got an unknown argument
        :raises: :class: `TypeError` : `segment_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(segment_id, int):
            raise TypeError(
                f"`segment_id` type is not valid. Expected `int`, got {type(segment_id)}."
            )

        if not isinstance(segment_id, int):
            raise TypeError("Segment ID are not valid.")

        available_params = ["filter", "limit", "after"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)
            query_params[key] = val

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{segment_id}", query_params
        ).json()

    def update(self, segment_id, name):
        """
        Update a segment

        Provides ability to update a segment.
        Ref: https://developers.mailerlite.com/docs/segments.html#update-segment

        :param segment_id: int Segment ID
        :param name: str Maximum length of 255 characters
        :raises: :class: `ValueError` : `name` cannot exceed 255 characters
        :raises: :class: `TypeError` : `segment_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(segment_id, int):
            raise TypeError(
                f"`segment_id` type is not valid. Expected `int`, got {type(segment_id)}."
            )

        if len(name) > 255:
            raise ValueError("`name` cannot exceed 255 characters.")

        params = locals()
        body_params = {"name": name}

        response = self.api_client.request(
            "PUT", f"{self.base_api_url}/{segment_id}", body=body_params
        )

        return True if response.status_code == 200 else False

    def delete(self, segment_id):
        """
        Delete a segment

        Provides ability to delete a segment.
        Ref: https://developers.mailerlite.com/docs/segments.html#delete-segment

        :param segment_id: int Segment ID
        :raises: :class: `TypeError` : `segment_id` type is not valid
        :return: `true` if action was successful, `false` if form was not found
        :rtype: bool
        """

        if not isinstance(segment_id, int):
            raise TypeError(
                f"`segment_id` type is not valid. Expected `int`, got {type(segment_id)}."
            )

        response = self.api_client.request(
            "DELETE", f"{self.base_api_url}/{segment_id}"
        )

        return True if response.status_code == 204 else False



from __future__ import absolute_import


class Groups(object):
    # Groups base API uri
    base_api_url = "api/groups"

    def __init__(self, api_client):
        self.api_client = api_client

    def list(self, **kwargs):
        """
        Lists all groups

        Returns a list of all groups.
        Ref: https://developers.mailerlite.com/docs/groups.html#list-all-groups

        :param **kwargs: You can pass additional arguments - page, limit, sort or to filter by name
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = ["list", "limit", "page", "sort"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)
            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request("GET", self.base_api_url, query_params).json()

    def create(self, name):
        """
        Create a group

        Provides an ability to create a new group.
        Ref: https://developers.mailerlite.com/docs/groups.html#create-a-group

        :param name: str Maximum length of 255 characters
        :raises: :class: `ValueError` : `name` cannot exceed 255 characters
        :return: JSON array
        :rtype: dict
        """

        if len(name) > 255:
            raise ValueError("`name` cannot exceed 255 characters.")

        params = locals()
        body_params = {"name": name}

        return self.api_client.request(
            "POST", self.base_api_url, body=body_params
        ).json()

    def update(self, group_id, name):
        """
        Update a group

        Provides an ability to update a group.
        Ref: https://developers.mailerlite.com/docs/groups.html#update-a-group

        :param group_id: int Group ID
        :param name: str Maximum length of 255 characters
        :raises: :class: `ValueError` : `name` cannot exceed 255 characters
        :raises: :class: `TypeError` : `group_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(group_id, int):
            raise TypeError(
                f"`group_id` type is not valid. Expected `int`, got {type(group_id)}."
            )

        if len(name) > 255:
            raise ValueError("Group name cannot exceed 255 characters.")

        params = locals()
        body_params = {"name": name}

        return self.api_client.request(
            "PUT", f"{self.base_api_url}/{group_id}", body=body_params
        ).json()

    def delete(self, group_id):
        """
        Delete a group

        Provides an ability to delete a group.
        Ref: https://developers.mailerlite.com/docs/groups.html#delete-group

        :param group_id: int Group ID
        :raises: :class: `TypeError` : `group_id` type is not valid
        :return: `true` if action was successful, `false` if form was not found
        :rtype: bool
        """

        if not isinstance(group_id, int):
            raise TypeError(
                f"`group_id` type is not valid. Expected `int`, got {type(group_id)}."
            )

        response = self.api_client.request("DELETE", f"{self.base_api_url}/{group_id}")

        return True if response.status_code == 204 else False

    def get_group_subscribers(self, group_id, **kwargs):
        """
        Get subscribers belonging to a group

        Returns a list of all subscribers belonging to a group.
        Ref: https://developers.mailerlite.com/docs/groups.html#get-subscribers-belonging-to-a-group

        :param group_id: int Group ID
        :param **kwargs: You can pass additional arguments - page, limit, sort or to filter by status
        :raises: :class: `TypeError` : Got an unknown argument
        :raises: :class: `TypeError` : `group_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(group_id, int):
            raise TypeError(
                f"`group_id` type is not valid. Expected `int`, got {type(group_id)}."
            )

        available_params = ["filter", "limit", "page"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)
            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request(
            "GET",
            f"{self.base_api_url}/{group_id}/subscribers",
            query_params,
        ).json()



from __future__ import absolute_import

# import all api files into api package
from mailerlite.sdk.campaigns import Campaigns
from mailerlite.sdk.fields import Fields
from mailerlite.sdk.forms import Forms
from mailerlite.sdk.groups import Groups
from mailerlite.sdk.segments import Segments
from mailerlite.sdk.subscribers import Subscribers
from mailerlite.sdk.timezones import Timezones
from mailerlite.sdk.webhooks import Webhooks



from __future__ import absolute_import


class Campaigns(object):
    # Campaigns base API uri
    base_api_url = "api/campaigns"

    def __init__(self, api_client):
        self.api_client = api_client

    def create(self, campaign):
        """
        Create a campaign
        Ref: https://developers.mailerlite.com/docs/campaigns.html#create-a-campaign

        :param campaign: dict A dict that contains all campaign related arguments
        :return: JSON array
        :rtype: dict
        """

        return self.api_client.request("POST", self.base_api_url, body=campaign).json()

    def update(self, campaign_id, campaign):
        """
        Update campaign

        Provides ability to update single campaign.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#update-campaign

        :param campaign_id: int Id of the campaign
        :param campaign: dict A dict that contains all campaign related arguments
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(campaign_id, int):
            raise TypeError(
                f"`campaign_id` type is not valid. Expected `int`, got {type(campaign_id)}."
            )

        return self.api_client.request(
            "PUT", f"{self.base_api_url}/{campaign_id}", body=campaign
        ).json()

    def get(self, campaign_id):
        """
        Get a campaign

        Returns information about single campaign.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#get-a-campaign

        :param campaign_id: int Id of the campaign
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(campaign_id, int):
            raise TypeError(
                f"`campaign_id` type is not valid. Expected `int`, got {type(campaign_id)}."
            )

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{campaign_id}"
        ).json()

    def list(self, **kwargs):
        """
        Campaign list

        Returns a list of all campaigns.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#campaign-list

        :param **kwargs: dict Allows you to pass additional arguments as query params (filter, limit and page)
        :return: JSON array
        :rtype: dict
        """

        available_params = ["filter", "page", "limit"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)

            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request("GET", self.base_api_url, query_params).json()

    def schedule(self, campaign_id, schedule):
        """
        Schedule a campaign

        Provides with ability to schedule existing campaign.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#schedule-a-campaign

        :param campaign_id: int Id of the campaign
        :param schedule: dict A dict with relevant parametes for scheduling campaign
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(campaign_id, int):
            raise TypeError(
                f"`campaign_id` type is not valid. Expected `int`, got {type(campaign_id)}."
            )

        return self.api_client.request(
            "POST",
            f"{self.base_api_url}/{campaign_id}/schedule",
            body=schedule,
        ).json()

    def cancel(self, campaign_id):
        """
        Cancel a ready campaign

        Provides with ability to cancel a campaign that is currently in a ready state.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#cancel-a-ready-campaign

        :param campaign_id: int Id of the campaign
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(campaign_id, int):
            raise TypeError(
                f"`campaign_id` type is not valid. Expected `int`, got {type(campaign_id)}."
            )

        return self.api_client.request(
            "POST", f"{self.base_api_url}/{campaign_id}/cancel"
        ).json()

    def delete(self, campaign_id):
        """
        Delete a campaign

        Provides with ability to delete existing campaign.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#delete-a-campaign

        :param campaign_id: int Id of the campaign
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(campaign_id, int):
            raise TypeError(
                f"`campaign_id` type is not valid. Expected `int`, got {type(campaign_id)}."
            )

        response = self.api_client.request(
            "DELETE", f"{self.base_api_url}/{campaign_id}"
        )

        return True if response.status_code == 204 else False

    def activity(self, campaign_id):
        """
        Get subscribers' activity of a sent campaign

        Returns subscribers' activity of a campaign.
        Ref: https://developers.mailerlite.com/docs/campaigns.html#get-subscribers-activity-of-a-sent-campaign

        :param campaign_id: int Id of the campaign
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(campaign_id, int):
            raise TypeError(
                f"`campaign_id` type is not valid. Expected `int`, got {type(campaign_id)}."
            )

        return self.api_client.request(
            "POST", f"{self.base_api_url}/{campaign_id}/reports/subscriber-activity"
        ).json()

    def languages(self):
        """
        Campaign languages

        Returns a list of all campaign languages available.
        Ref: https://developers.mailerlite.com/docs/campaign-languages.html#response

        :return: JSON array
        :rtype: dict
        """

        return self.api_client.request("GET", f"{self.base_api_url}/languages").json()



from __future__ import absolute_import

import datetime
import json

import requests


class ApiClient(object):
    def __init__(self, config={}):
        """Contructor"""

        # Base URL of the API
        self.host = "https://connect.mailerlite.com/"

        # Base headers
        self.default_headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "MailerLite-Python-SDK-Client",
        }
        self.set_config(config)

    def set_config(self, config={}):
        """API Client Configuration Setter"""

        # Authentication
        self.api_key = config["api_key"] if "api_key" in config.keys() else ""
        self.default_headers["Authorization"] = "Bearer " + self.api_key

        # API version
        if "api_version" in config.keys():
            self.api_version = config["api_version"]
            self.default_headers["X-Version"] = self.api_version

        # Request timeout
        self.timeout = config["timeout"] if "timeout" in config.keys() else 120

    def request(self, method, url, query_params=None, body=None):
        """Requests Wrapper"""

        if method == "POST":
            return requests.post(
                self.host + url,
                data=json.dumps(body),
                params=query_params,
                headers=self.default_headers,
                timeout=self.timeout,
            )
        elif method == "GET":
            return requests.get(
                self.host + url,
                params=query_params,
                headers=self.default_headers,
                timeout=self.timeout,
            )
        elif method == "PUT":
            return requests.put(
                self.host + url,
                data=json.dumps(body),
                params=query_params,
                headers=self.default_headers,
                timeout=self.timeout,
            )
        elif method == "DELETE":
            return requests.delete(
                self.host + url,
                params=query_params,
                headers=self.default_headers,
                timeout=self.timeout,
            )
        else:
            raise ValueError("http method must be `POST`, `GET`, `PUT` or `DELETE`.")



from __future__ import absolute_import


class Forms(object):
    # Forms base API uri
    base_api_url = "api/forms"

    def __init__(self, api_client):
        self.api_client = api_client

    def list(self, type, **kwargs):
        """
        Lists all automations on account
        Ref: https://developers.mailerlite.com/docs/forms.html#list-all-forms

        :param **kwargs: You can pass additional arguments - page, limit, sort or to filter by name
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = ["limit", "page", "filter", "sort"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)

            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{type}", query_params
        ).json()

    def get(self, form_id):
        """
        Get a form

        Returns information about single form.
        Ref: https://developers.mailerlite.com/docs/forms.html#get-a-form

        :param form_id: int Form ID
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(form_id, int):
            raise TypeError(
                f"`form_id` type is not valid. Expected `int`, got {type(form_id)}."
            )

        return self.api_client.request("GET", f"{self.base_api_url}/{form_id}").json()

    def update(self, form_id, name):
        """
        Update a form

        Provides ability to update existing form.
        Ref: https://developers.mailerlite.com/docs/forms.html#update-a-form

        :param form_id: int Form ID
        :param name: string Maximum length of 255 characters
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(form_id, int):
            raise TypeError(
                f"`form_id` type is not valid. Expected `int`, got {type(form_id)}."
            )

        body_params = {"name": name}

        return self.api_client.request(
            "PUT", f"{self.base_api_url}/{form_id}", body=body_params
        ).json()

    def get_subscribers(self, form_id, **kwargs):
        """
        Get subscribers who signed up to a specific form

        Provides a list of subscribers who signed up to a specific form.
        Ref: https://developers.mailerlite.com/docs/forms.html#get-subscribers-who-signed-up-to-a-specific-form

        :param form_id: int Form ID
        :param **kwargs: dict You can pass additional arguments - page, limit or to filter by status
        :raises: :class: `TypeError` : Got an unknown argument
        :return: `true` if action was successful, `false` if field was not found
        :rtype: bool
        """

        if not isinstance(form_id, int):
            raise TypeError(
                f"`form_id` type is not valid. Expected `int`, got {type(form_id)}."
            )

        available_params = ["limit", "page", "filter"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)

            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{form_id}/subscribers", query_params
        ).json()

    def delete(self, form_id):
        """
        Delete a form

        Provides ability to delete a form.
        Ref: https://developers.mailerlite.com/docs/forms.html#delete-a-form

        :param form_id: int Form ID
        :return: `true` if action was successful, `false` if form was not found
        :rtype: bool
        """

        if not isinstance(form_id, int):
            raise TypeError(
                f"`form_id` type is not valid. Expected `int`, got {type(form_id)}."
            )

        response = self.api_client.request("DELETE", f"{self.base_api_url}/{form_id}")

        return True if response.status_code == 204 else False



from __future__ import absolute_import


class Batches(object):
    base_api_url = "api/batch"

    def __init__(self, api_client):
        self.api_client = api_client

    def request(self, requests=[]):
        """
        Send a batch request

        Provides an ability to make multiple request to our api in a single call.
        Ref: https://developers.mailerlite.com/docs/batching.html#request-body

        :param requests: dict Array of objects containing required method and path properties and optional body
        :raises: :class: `TypeError` : `requests` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if type(requests) is not list:
            raise TypeError(
                f"`requests` type is not valid. Expected `dict`, got {type(requests)}."
            )

        body_params = {"requests": requests}

        return self.api_client.request(
            "POST", self.base_api_url, body=body_params
        ).json()



from __future__ import absolute_import


class Automations(object):
    # Automations base API uri
    base_api_url = "api/automations"

    def __init__(self, api_client):
        """
        Class constructor

        :param api_client: object
        """
        self.api_client = api_client

    def list(self, **kwargs):
        """
        Lists all automations on account
        Ref: https://developers.mailerlite.com/docs/automations.html#list-all-automations

        :param **kwargs: You can pass additional arguments - page, limit or to filter by status, name and group
        :return: JSON array
        :rtype: dict
        """

        return self.api_client.request("GET", self.base_api_url).json()

    def get(self, automation_id):
        """
        Returns information about single automation
        Ref: https://developers.mailerlite.com/docs/automations.html#get-an-automation

        :param: automation_id: Automation ID
        :return: JSON dict
        """

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{automation_id}"
        ).json()

    def activity(self, automation_id, **kwargs):
        """
        Get the subscriber activity for an automation
        Ref: https://developers.mailerlite.com/docs/automations.html#get-the-subscriber-activity-for-an-automation

        :param: automation_id: Automation ID
        :param: kwargs: You can pass additional arguments - page, limit or to filter by status, date_from, date_to
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON dict
        """

        available_params = ["filter", "page", "limit"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)

            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request(
            "GET",
            f"{self.base_api_url}/{automation_id}/activity",
            query_params,
        ).json()



from __future__ import absolute_import


class Timezones(object):
    # Timezones base API uri
    base_api_url = "api/timezones"

    def __init__(self, api_client):
        self.api_client = api_client

    def list(self):
        """
        Lists all timezones

        Returns a list of all timezones.
        Ref: https://developers.mailerlite.com/docs/timezones.html#response

        :return: JSON array
        :rtype: dict
        """

        return self.api_client.request("GET", self.base_api_url).json()



from __future__ import absolute_import


class Fields(object):
    # Fields base API uri
    base_api_url = "api/fields"

    def __init__(self, api_client):
        self.api_client = api_client

    def create(self, name, type):
        """
        Create a field

        Provides ability to createa field.
        Ref: https://developers.mailerlite.com/docs/fields.html#create-a-field

        :param name: string Field name. Maximum length of 255 characters.
        :param type: string Can be text, number or date.
        :raises: :class: `ValueError` : Type be text, number or date
        :raises: :class: `ValueError` : `name` cannot exceed 255 characters
        :return: JSON array
        :rtype: dict
        """

        allowed_types = ["text", "number", "date"]

        if type not in allowed_types:
            raise ValueError("`type` can be text, number or date.")

        if len(name) > 255:
            raise ValueError("`name` cannot exceed 255 characters.")

        body_params = {"name": name, "type": type}

        return self.api_client.request(
            "POST", self.base_api_url, body=body_params
        ).json()

    def list(self, **kwargs):
        """
        List all fields

        Returns a list of all fields.
        Ref: https://developers.mailerlite.com/docs/fields.html#list-all-fields

        :param **kwargs: dict You can pass additional arguments such as `list`, `page`, `filter` and `sort.
        :raises: :class: `TypeError` : Got an unknown argument
        :return: JSON array
        :rtype: dict
        """

        available_params = ["limit", "page", "filter", "sort"]

        params = locals()
        query_params = {}
        for key, val in params["kwargs"].items():
            if key not in available_params:
                raise TypeError("Got an unknown argument '%s'" % key)
            if key == "filter":
                for filter_key, filter_value in val.items():
                    query_params[filter_key] = filter_value
            else:
                query_params[key] = val

        return self.api_client.request("GET", self.base_api_url, query_params).json()

    def update(self, field_id, name):
        """
        Update a field

        Provides the ability to update a field.
        Ref: https://developers.mailerlite.com/docs/fields.html#update-a-field

        :param field_id: int Field ID.
        :param name: string Can be text, number or date.
        :raises: :class: `ValueError` : `name` cannot exceed 255 characters
        :raises: :class: `TypeError` : `field_id` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(field_id, int):
            raise TypeError(
                f"`field_id` type is not valid. Expected `int`, got {type(field_id)}."
            )

        if len(name) > 255:
            raise ValueError("Field name cannot exceed 255 characters.")

        body_params = {"name": name}

        return self.api_client.request(
            "PUT", f"{self.base_api_url}/{field_id}", body=body_params
        ).json()

    def delete(self, field_id):
        """
        Delete a field

        Provides the ability to delete a field.
        Ref: https://developers.mailerlite.com/docs/fields.html#update-a-field

        :param field_id: int Field ID.
        :return: `true` if action was successful, `false` if field was not found
        :rtype: bool
        """

        response = self.api_client.request("DELETE", f"{self.base_api_url}/{field_id}")

        return True if response.status_code == 204 else False



from __future__ import absolute_import


class Webhooks(object):
    # Webhooks base API uri
    base_api_url = "api/webhooks"

    def __init__(self, api_client):
        self.api_client = api_client

    def list(self):
        """
        Lists all webhooks

        Returns a list of all webhooks.
        Ref: https://developers.mailerlite.com/docs/webhooks.html#list-all-webhooks

        :return: JSON array
        :rtype: dict
        """

        return self.api_client.request("GET", self.base_api_url).json()

    def get(self, webhook_id):
        """
        Get a webhook

        Returns information about single webhook.
        Ref: https://developers.mailerlite.com/docs/webhooks.html#get-a-webhook

        :param webhook_id: int Webhook ID
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(webhook_id, int):
            raise TypeError(
                f"`webhook_id` type is not valid. Expected `int`, got {type(webhook_id)}."
            )

        return self.api_client.request(
            "GET", f"{self.base_api_url}/{webhook_id}"
        ).json()

    def update(self, webhook_id, events=None, url=None, name=None, enabled=True):
        """
        Update a webhook

        Provides ability to update an existing webhook.
        Ref: https://developers.mailerlite.com/docs/webhooks.html#update-a-webhook

        :param webhook_id: int Webhook ID
        :param events: list A list of events https://developers.mailerlite.com/docs/webhooks.html#available-events
        :param url: str Webhook URL
        :param name: str Webhook name
        :param enabled: bool Defines the state of a webhook
        :return: JSON array
        :rtype: dict
        """

        if not isinstance(webhook_id, int):
            raise TypeError(
                f"`webhook_id` type is not valid. Expected `int`, got {type(webhook_id)}."
            )

        if type(events) is not list and events is not None:
            raise TypeError(
                f"`events` type is not valid. Expected `list`, got {type(events)}."
            )

        params = locals()
        body_params = {}
        for key, val in params.items():
            if val is not None and key not in ["self", "webhook_id"]:
                body_params[key] = val

        return self.api_client.request(
            "PUT", f"{self.base_api_url}/{webhook_id}", body=body_params
        ).json()

    def create(self, events, url, name=None):
        """
        Create a webhook

        Provides ability to create a new webhook.
        Ref: https://developers.mailerlite.com/docs/webhooks.html#create-a-webhook

        :param events: list A list of events https://developers.mailerlite.com/docs/webhooks.html#available-events
        :param url: str Webhook URL
        :param name: str Webhook name
        :raises: :class: `TypeError` : `events` type is not valid
        :return: JSON array
        :rtype: dict
        """

        if type(events) is not list:
            raise TypeError(
                f"`events` type is not valid. Expected `list`, got {type(events)}."
            )

        body_params = {"events": events, "url": url}
        if len(name) > 0:
            body_params["name"] = name

        return self.api_client.request(
            "POST", self.base_api_url, body=body_params
        ).json()

    def delete(self, webhook_id):
        """
        Delete a webhook

        Provides ability to delete existing webhook.
        Ref: https://developers.mailerlite.com/docs/webhooks.html#delete-a-webhook

        :param webhook_id: int Webhook URL
        :raises: :class: `TypeError` : `webhook_id` type is not valid
        :return: `true` if action was successful, `false` if form was not found
        :rtype: bool
        """

        if not isinstance(webhook_id, int):
            raise TypeError(
                f"`webhook_id` type is not valid. Expected `int`, got {type(webhook_id)}."
            )

        response = self.api_client.request(
            "DELETE", f"{self.base_api_url}/{webhook_id}"
        )

        return True if response.status_code == 204 else False



# coding: utf-8

from __future__ import absolute_import

from mailerlite.api_client import ApiClient
from mailerlite.sdk.automations import Automations
from mailerlite.sdk.batch import Batches
# import all api files into api package
from mailerlite.sdk.campaigns import Campaigns
from mailerlite.sdk.fields import Fields
from mailerlite.sdk.forms import Forms
from mailerlite.sdk.groups import Groups
from mailerlite.sdk.segments import Segments
from mailerlite.sdk.subscribers import Subscribers
from mailerlite.sdk.timezones import Timezones
from mailerlite.sdk.webhooks import Webhooks


class Client(object):
    def __init__(self, config={}):
        # Initialize ApiClient
        self.api_client = ApiClient(config)

        # Bootstrap API classes
        self.subscribers = Subscribers(self.api_client)
        self.groups = Groups(self.api_client)
        self.segments = Segments(self.api_client)
        self.fields = Fields(self.api_client)
        self.forms = Forms(self.api_client)
        self.campaigns = Campaigns(self.api_client)
        self.timezones = Timezones(self.api_client)
        self.webhooks = Webhooks(self.api_client)
        self.automations = Automations(self.api_client)
        self.batches = Batches(self.api_client)
